# -*- coding: utf-8 -*-

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

import sys
import unittest


_PY2 = sys.version_info[0] == 2


class GorillaTestCase(unittest.TestCase):

    if _PY2:
        assertCountEqual = unittest.TestCase.assertItemsEqual



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

import importlib
import os
import sys

_HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(_HERE, os.pardir)))

import gorilla

import tests.decorators as _decorators
import tests.decorators.frommodule as _frommodule
import tests.decorators.tomodule as _tomodule
from tests._testcase import GorillaTestCase


_MODULES = [
    ('_decorators', _decorators.__name__),
    ('_frommodule', _frommodule.__name__),
    ('_tomodule', _tomodule.__name__),
]


class DecoratorsTest(GorillaTestCase):

    def setUp(self):
        for module, path in _MODULES:
            globals()[module] = importlib.import_module(path)

    def tearDown(self):
        for module, path in _MODULES:
            if path in sys.modules:
                del sys.modules[path]

    def test_patch_decorator_on_function(self):
        destination = _tomodule
        obj = gorilla.get_attribute(_frommodule, 'function')

        self.assertIs(gorilla.patch(destination)(obj), obj)

        decorator_data = gorilla.get_decorator_data(obj)
        expected_patches = [
            gorilla.Patch(destination, 'function', obj),
        ]
        self.assertEqual(decorator_data.patches, expected_patches)

    def test_patch_decorator_on_class(self):
        destination = _tomodule
        obj = _frommodule.Class

        self.assertIs(gorilla.patch(destination)(obj), obj)

        decorator_data = gorilla.get_decorator_data(obj)
        expected_patches = [
            gorilla.Patch(destination, 'Class', obj),
        ]
        self.assertEqual(decorator_data.patches, expected_patches)

    def test_patch_decorator(self):
        destination = _tomodule
        obj = gorilla.get_attribute(_frommodule, 'function')

        settings = gorilla.Settings(allow_hit=True, store_hit=True)
        self.assertIs(gorilla.patch(destination, settings=settings)(obj), obj)
        settings.allow_hit = False
        settings.store_hit = False

        decorator_data = gorilla.get_decorator_data(obj)
        expected_patches = [
            gorilla.Patch(destination, 'function', obj, gorilla.Settings(allow_hit=True, store_hit=True)),
        ]
        self.assertEqual(decorator_data.patches, expected_patches)

    def test_patches_decorator_on_class(self):
        destination = _tomodule.Class
        obj = _frommodule.Class

        self.assertIs(gorilla.patches(destination)(obj), obj)

        decorator_data = gorilla.get_decorator_data(obj)
        expected_patches = [
            gorilla.Patch(destination, 'STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            gorilla.Patch(destination, 'class_method', gorilla.get_attribute(obj, 'class_method')),
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
            gorilla.Patch(destination, 'static_method', gorilla.get_attribute(obj, 'static_method')),
            gorilla.Patch(destination, 'value', gorilla.get_attribute(obj, 'value')),
            gorilla.Patch(destination.Inner, 'STATIC_VALUE', gorilla.get_attribute(obj.Inner, 'STATIC_VALUE')),
            gorilla.Patch(destination.Inner, 'method', gorilla.get_attribute(obj.Inner, 'method')),
        ]
        self.assertEqual(decorator_data.patches, expected_patches)

    def test_patches_decorator(self):
        destination = _tomodule.Class
        obj = _frommodule.Class

        settings = gorilla.Settings(allow_hit=True, store_hit=True)
        self.assertIs(gorilla.patches(destination, settings=settings)(obj), obj)
        settings.allow_hit = False
        settings.store_hit = False

        decorator_data = gorilla.get_decorator_data(obj)
        expected_patches = [
            gorilla.Patch(destination, 'STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE'), settings=gorilla.Settings(allow_hit=True, store_hit=True)),
            gorilla.Patch(destination, 'class_method', gorilla.get_attribute(obj, 'class_method'), settings=gorilla.Settings(allow_hit=True, store_hit=True)),
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method'), settings=gorilla.Settings(allow_hit=True, store_hit=True)),
            gorilla.Patch(destination, 'static_method', gorilla.get_attribute(obj, 'static_method'), settings=gorilla.Settings(allow_hit=True, store_hit=True)),
            gorilla.Patch(destination, 'value', gorilla.get_attribute(obj, 'value'), settings=gorilla.Settings(allow_hit=True, store_hit=True)),
            gorilla.Patch(destination.Inner, 'STATIC_VALUE', gorilla.get_attribute(obj.Inner, 'STATIC_VALUE'), settings=gorilla.Settings(allow_hit=True, store_hit=True)),
            gorilla.Patch(destination.Inner, 'method', gorilla.get_attribute(obj.Inner, 'method'), settings=gorilla.Settings(allow_hit=True, store_hit=True)),
        ]
        self.assertEqual(decorator_data.patches, expected_patches)

    def test_destination_decorator(self):
        destination = _tomodule.Class
        obj = _frommodule.Class

        destination_override = _tomodule.Parent
        gorilla.destination(destination_override)(gorilla.get_attribute(obj, 'class_method'))
        gorilla.destination(destination_override)(gorilla.get_attribute(obj, 'method'))
        gorilla.destination(destination_override)(gorilla.get_attribute(obj, 'value'))
        gorilla.patches(destination)(obj)

        decorator_data = gorilla.get_decorator_data(obj)
        expected_patches = [
            gorilla.Patch(destination, 'STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            gorilla.Patch(destination_override, 'class_method', gorilla.get_attribute(obj, 'class_method')),
            gorilla.Patch(destination_override, 'method', gorilla.get_attribute(obj, 'method')),
            gorilla.Patch(destination, 'static_method', gorilla.get_attribute(obj, 'static_method')),
            gorilla.Patch(destination_override, 'value', gorilla.get_attribute(obj, 'value')),
            gorilla.Patch(destination.Inner, 'STATIC_VALUE', gorilla.get_attribute(obj.Inner, 'STATIC_VALUE')),
            gorilla.Patch(destination.Inner, 'method', gorilla.get_attribute(obj.Inner, 'method')),
        ]
        self.assertEqual(decorator_data.patches, expected_patches)

    def test_name_decorator(self):
        destination = _tomodule.Class
        obj = _frommodule.Class

        name_override = 'whatever'
        gorilla.name(name_override)(gorilla.get_attribute(obj, 'class_method'))
        gorilla.name(name_override)(gorilla.get_attribute(obj, 'static_method'))
        gorilla.name(name_override)(gorilla.get_attribute(obj.Inner, 'method'))
        gorilla.patches(destination)(obj)

        decorator_data = gorilla.get_decorator_data(obj)
        expected_patches = [
            gorilla.Patch(destination, 'STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            gorilla.Patch(destination, name_override, gorilla.get_attribute(obj, 'class_method')),
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
            gorilla.Patch(destination, name_override, gorilla.get_attribute(obj, 'static_method')),
            gorilla.Patch(destination, 'value', gorilla.get_attribute(obj, 'value')),
            gorilla.Patch(destination.Inner, 'STATIC_VALUE', gorilla.get_attribute(obj.Inner, 'STATIC_VALUE')),
            gorilla.Patch(destination.Inner, name_override, gorilla.get_attribute(obj.Inner, 'method')),
        ]
        self.assertEqual(decorator_data.patches, expected_patches)

    def test_settings_decorator_1(self):
        destination = _tomodule.Class
        obj = _frommodule.Class

        gorilla.settings(some_value=123)(gorilla.get_attribute(obj, 'method'))
        gorilla.settings(allow_hit=True)(gorilla.get_attribute(obj, 'static_method'))
        gorilla.settings(store_hit=False)(gorilla.get_attribute(obj, 'value'))
        gorilla.settings(allow_hit=True, store_hit=False)(gorilla.get_attribute(obj.Inner, 'method'))
        gorilla.patches(destination)(obj)

        decorator_data = gorilla.get_decorator_data(obj)
        expected_patches = [
            gorilla.Patch(destination, 'STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            gorilla.Patch(destination, 'class_method', gorilla.get_attribute(obj, 'class_method')),
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method'), settings=gorilla.Settings(some_value=123)),
            gorilla.Patch(destination, 'static_method', gorilla.get_attribute(obj, 'static_method'), settings=gorilla.Settings(allow_hit=True)),
            gorilla.Patch(destination, 'value', gorilla.get_attribute(obj, 'value'), settings=gorilla.Settings(store_hit=False)),
            gorilla.Patch(destination.Inner, 'STATIC_VALUE', gorilla.get_attribute(obj.Inner, 'STATIC_VALUE')),
            gorilla.Patch(destination.Inner, 'method', gorilla.get_attribute(obj.Inner, 'method'), settings=gorilla.Settings(allow_hit=True, store_hit=False)),
        ]
        self.assertEqual(decorator_data.patches, expected_patches)

    def test_settings_decorator_2(self):
        destination = _tomodule.Class
        obj = _frommodule.Class

        gorilla.settings(some_value=123)(gorilla.get_attribute(obj, 'method'))
        gorilla.settings(allow_hit=False)(gorilla.get_attribute(obj, 'static_method'))
        gorilla.settings(store_hit=True)(gorilla.get_attribute(obj, 'value'))
        gorilla.settings(allow_hit=False, store_hit=True)(gorilla.get_attribute(obj.Inner, 'method'))
        gorilla.patches(destination, settings=gorilla.Settings(allow_hit=True, store_hit=False))(obj)

        decorator_data = gorilla.get_decorator_data(obj)
        expected_patches = [
            gorilla.Patch(destination, 'STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE'), settings=gorilla.Settings(allow_hit=True, store_hit=False)),
            gorilla.Patch(destination, 'class_method', gorilla.get_attribute(obj, 'class_method'), settings=gorilla.Settings(allow_hit=True, store_hit=False)),
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method'), settings=gorilla.Settings(allow_hit=True, some_value=123, store_hit=False)),
            gorilla.Patch(destination, 'static_method', gorilla.get_attribute(obj, 'static_method'), settings=gorilla.Settings(store_hit=False)),
            gorilla.Patch(destination, 'value', gorilla.get_attribute(obj, 'value'), settings=gorilla.Settings(allow_hit=True)),
            gorilla.Patch(destination.Inner, 'STATIC_VALUE', gorilla.get_attribute(obj.Inner, 'STATIC_VALUE'), settings=gorilla.Settings(allow_hit=True, store_hit=False)),
            gorilla.Patch(destination.Inner, 'method', gorilla.get_attribute(obj.Inner, 'method'), settings=gorilla.Settings(allow_hit=False, store_hit=True)),
        ]
        self.assertEqual(decorator_data.patches, expected_patches)

    def test_filter_decorator(self):
        destination = _tomodule.Class
        obj = _frommodule.Class

        gorilla.filter(True)(gorilla.get_attribute(obj, '__init__'))
        gorilla.filter(False)(gorilla.get_attribute(obj, 'class_method'))
        gorilla.filter(False)(gorilla.get_attribute(obj.Inner, 'method'))
        gorilla.patches(destination)(obj)

        decorator_data = gorilla.get_decorator_data(obj)
        expected_patches = [
            gorilla.Patch(destination, 'STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            gorilla.Patch(destination, '__init__', gorilla.get_attribute(obj, '__init__')),
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
            gorilla.Patch(destination, 'static_method', gorilla.get_attribute(obj, 'static_method')),
            gorilla.Patch(destination, 'value', gorilla.get_attribute(obj, 'value')),
            gorilla.Patch(destination.Inner, 'STATIC_VALUE', gorilla.get_attribute(obj.Inner, 'STATIC_VALUE')),
        ]
        self.assertEqual(decorator_data.patches, expected_patches)


if __name__ == '__main__':
    from tests.run import run
    run('__main__')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

import argparse
import collections
import os
import sys
import unittest


# Usage's syntax based on docopt.
_USAGE = "%(prog)s [<name>...]"
_DESCRIPTION = (
    "Runs the tests that have their name containing either one of the 'name' "
    "arguments passed. If no 'name' argument is passed, all the tests are run."
)


def _find_tests(path, selectors=None):
    if selectors is None:
        def filter(test):
            return True
    else:
        def filter(test):
            return any(selector in _get_test_full_name(test)
                       for selector in selectors)

    out = []
    loader = unittest.TestLoader()
    if path == '__main__':
        root_test = loader.loadTestsFromModule(sys.modules[path])
    else:
        root_test = loader.discover(path)

    stack = collections.deque((root_test,))
    while stack:
        obj = stack.popleft()
        if isinstance(obj, unittest.TestSuite):
            stack.extend(test for test in obj)
        elif type(obj).__name__ == 'ModuleImportFailure':
            try:
                # This should always throw an ImportError exception.
                getattr(obj, _get_test_name(obj))()
            except ImportError as e:
                sys.exit(e.message.strip())
        elif filter(obj):
            out.append(obj)

    return out


def _get_test_name(test):
    return test._testMethodName


def _get_test_full_name(test):
    return '%s.%s.%s' % (type(test).__module__, type(test).__name__,
                         _get_test_name(test))


def run(start_path, verbosity=2):
    parser = argparse.ArgumentParser(usage=_USAGE, description=_DESCRIPTION)
    parser.add_argument('name', nargs='*',
                        help='partial test names to search')
    args = parser.parse_args()
    selectors = args.name if args.name else None
    tests = _find_tests(start_path, selectors)
    suite = unittest.TestLoader().suiteClass(tests)
    unittest.TextTestRunner(verbosity=verbosity).run(suite)


if __name__ == "__main__":
    run(os.path.abspath(os.path.dirname(__file__)))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

import importlib
import os
import sys

_HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(_HERE, os.pardir)))

import gorilla

import tests.utils as _utils
import tests.utils.frommodule as _frommodule
import tests.utils.tomodule as _tomodule
import tests.utils.subpackage as _subpackage
import tests.utils.subpackage.module1 as _module1
import tests.utils.subpackage.module2 as _module2
from tests._testcase import GorillaTestCase


if sys.version_info[0] == 2:
    def _iteritems(d, **kwargs):
        return d.iteritems(**kwargs)

    def _unfold(obj):
        return obj.__func__
else:
    def _iteritems(d, **kwargs):
        return iter(d.items(**kwargs))

    def _unfold(obj):
        return obj


_MODULES = [
    ('_utils', _utils.__name__),
    ('_frommodule', _frommodule.__name__),
    ('_tomodule', _tomodule.__name__),
    ('_subpackage', _subpackage.__name__),
    ('_module1', _module1.__name__),
    ('_module2', _module2.__name__),
]


class UtilsTest(GorillaTestCase):

    def setUp(self):
        for module, path in _MODULES:
            globals()[module] = importlib.import_module(path)

    def tearDown(self):
        for module, path in _MODULES:
            if path in sys.modules:
                del sys.modules[path]

    def test_create_patches_1(self):
        destination = _tomodule
        obj = _frommodule
        patches = gorilla.create_patches(destination, obj,
                                         use_decorators=False)
        expected_patches = [
            gorilla.Patch(destination, 'function', gorilla.get_attribute(obj, 'function')),
            gorilla.Patch(destination, 'global_variable', gorilla.get_attribute(obj, 'global_variable')),
            gorilla.Patch(destination, 'unbound_class_method', gorilla.get_attribute(obj, 'unbound_class_method')),
            gorilla.Patch(destination, 'unbound_method', gorilla.get_attribute(obj, 'unbound_method')),
            gorilla.Patch(destination, 'unbound_static_method', gorilla.get_attribute(obj, 'unbound_static_method')),
            gorilla.Patch(destination.Child, 'STATIC_VALUE', gorilla.get_attribute(obj.Child, 'STATIC_VALUE')),
            gorilla.Patch(destination.Child, 'child_value', gorilla.get_attribute(obj.Child, 'child_value')),
            gorilla.Patch(destination.Child, 'from_value', gorilla.get_attribute(obj.Child, 'from_value')),
            gorilla.Patch(destination.Child, 'instance_value', gorilla.get_attribute(obj.Child, 'instance_value')),
            gorilla.Patch(destination.Child, 'method', gorilla.get_attribute(obj.Child, 'method')),
            gorilla.Patch(destination.Child, 'parent_value', gorilla.get_attribute(obj.Child, 'parent_value')),
            gorilla.Patch(destination.Child, 'to_value', gorilla.get_attribute(obj.Child, 'to_value')),
            gorilla.Patch(destination.Class, 'STATIC_VALUE', gorilla.get_attribute(obj.Class, 'STATIC_VALUE')),
            gorilla.Patch(destination.Class, 'class_method', gorilla.get_attribute(obj.Class, 'class_method')),
            gorilla.Patch(destination.Class, 'method', gorilla.get_attribute(obj.Class, 'method')),
            gorilla.Patch(destination.Class, 'static_method', gorilla.get_attribute(obj.Class, 'static_method')),
            gorilla.Patch(destination.Class, 'value', gorilla.get_attribute(obj.Class, 'value')),
            gorilla.Patch(destination.Parent, 'STATIC_VALUE', gorilla.get_attribute(obj.Parent, 'STATIC_VALUE')),
            gorilla.Patch(destination.Parent, 'from_value', gorilla.get_attribute(obj.Parent, 'from_value')),
            gorilla.Patch(destination.Parent, 'instance_value', gorilla.get_attribute(obj.Parent, 'instance_value')),
            gorilla.Patch(destination.Parent, 'method', gorilla.get_attribute(obj.Parent, 'method')),
            gorilla.Patch(destination.Parent, 'parent_value', gorilla.get_attribute(obj.Parent, 'parent_value')),
            gorilla.Patch(destination.Parent, 'to_value', gorilla.get_attribute(obj.Parent, 'to_value')),
            gorilla.Patch(destination.Class.Inner, 'STATIC_VALUE', gorilla.get_attribute(obj.Class.Inner, 'STATIC_VALUE')),
            gorilla.Patch(destination.Class.Inner, 'method', gorilla.get_attribute(obj.Class.Inner, 'method')),
        ]
        self.assertEqual(patches, expected_patches)

        destination = _tomodule.Class
        obj = _frommodule.Class
        patches = gorilla.create_patches(destination, obj,
                                         use_decorators=False)
        expected_patches = [
            gorilla.Patch(destination, 'STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            gorilla.Patch(destination, 'class_method', gorilla.get_attribute(obj, 'class_method')),
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
            gorilla.Patch(destination, 'static_method', gorilla.get_attribute(obj, 'static_method')),
            gorilla.Patch(destination, 'value', gorilla.get_attribute(obj, 'value')),
            gorilla.Patch(destination.Inner, 'STATIC_VALUE', gorilla.get_attribute(obj.Inner, 'STATIC_VALUE')),
            gorilla.Patch(destination.Inner, 'method', gorilla.get_attribute(obj.Inner, 'method')),
        ]
        self.assertEqual(patches, expected_patches)

        destination = _tomodule.Parent
        obj = _frommodule.Parent
        patches = gorilla.create_patches(destination, obj,
                                         use_decorators=False)
        expected_patches = [
            gorilla.Patch(destination, 'STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            gorilla.Patch(destination, 'from_value', gorilla.get_attribute(obj, 'from_value')),
            gorilla.Patch(destination, 'instance_value', gorilla.get_attribute(obj, 'instance_value')),
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
            gorilla.Patch(destination, 'parent_value', gorilla.get_attribute(obj, 'parent_value')),
            gorilla.Patch(destination, 'to_value', gorilla.get_attribute(obj, 'to_value')),
        ]
        self.assertEqual(patches, expected_patches)

        destination = _tomodule.Child
        obj = _frommodule.Child
        patches = gorilla.create_patches(destination, obj,
                                         use_decorators=False)
        expected_patches = [
            gorilla.Patch(destination, 'STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            gorilla.Patch(destination, 'child_value', gorilla.get_attribute(obj, 'child_value')),
            gorilla.Patch(destination, 'from_value', gorilla.get_attribute(obj, 'from_value')),
            gorilla.Patch(destination, 'instance_value', gorilla.get_attribute(obj, 'instance_value')),
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
            gorilla.Patch(destination, 'parent_value', gorilla.get_attribute(obj, 'parent_value')),
            gorilla.Patch(destination, 'to_value', gorilla.get_attribute(obj, 'to_value')),
        ]
        self.assertEqual(patches, expected_patches)

    def test_create_patches_2(self):
        destination = _tomodule
        obj = _frommodule
        patches = gorilla.create_patches(destination, obj, recursive=False,
                                         use_decorators=False)
        expected_patches = [
            gorilla.Patch(destination, 'Child', gorilla.get_attribute(obj, 'Child')),
            gorilla.Patch(destination, 'Class', gorilla.get_attribute(obj, 'Class')),
            gorilla.Patch(destination, 'Parent', gorilla.get_attribute(obj, 'Parent')),
            gorilla.Patch(destination, 'function', gorilla.get_attribute(obj, 'function')),
            gorilla.Patch(destination, 'global_variable', gorilla.get_attribute(obj, 'global_variable')),
            gorilla.Patch(destination, 'unbound_class_method', gorilla.get_attribute(obj, 'unbound_class_method')),
            gorilla.Patch(destination, 'unbound_method', gorilla.get_attribute(obj, 'unbound_method')),
            gorilla.Patch(destination, 'unbound_static_method', gorilla.get_attribute(obj, 'unbound_static_method')),
        ]
        self.assertEqual(patches, expected_patches)

        destination = _tomodule.Class
        obj = _frommodule.Class
        patches = gorilla.create_patches(destination, obj, recursive=False,
                                         use_decorators=False)
        expected_patches = [
            gorilla.Patch(destination, 'Inner', gorilla.get_attribute(obj, 'Inner')),
            gorilla.Patch(destination, 'STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            gorilla.Patch(destination, 'class_method', gorilla.get_attribute(obj, 'class_method')),
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
            gorilla.Patch(destination, 'static_method', gorilla.get_attribute(obj, 'static_method')),
            gorilla.Patch(destination, 'value', gorilla.get_attribute(obj, 'value')),
        ]
        self.assertEqual(patches, expected_patches)

        destination = _tomodule.Parent
        obj = _frommodule.Parent
        patches = gorilla.create_patches(destination, obj, recursive=False,
                                         use_decorators=False)
        expected_patches = [
            gorilla.Patch(destination, 'STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            gorilla.Patch(destination, 'from_value', gorilla.get_attribute(obj, 'from_value')),
            gorilla.Patch(destination, 'instance_value', gorilla.get_attribute(obj, 'instance_value')),
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
            gorilla.Patch(destination, 'parent_value', gorilla.get_attribute(obj, 'parent_value')),
            gorilla.Patch(destination, 'to_value', gorilla.get_attribute(obj, 'to_value')),
        ]
        self.assertEqual(patches, expected_patches)

        destination = _tomodule.Child
        obj = _frommodule.Child
        patches = gorilla.create_patches(destination, obj, recursive=False,
                                         use_decorators=False)
        expected_patches = [
            gorilla.Patch(destination, 'STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            gorilla.Patch(destination, 'child_value', gorilla.get_attribute(obj, 'child_value')),
            gorilla.Patch(destination, 'from_value', gorilla.get_attribute(obj, 'from_value')),
            gorilla.Patch(destination, 'instance_value', gorilla.get_attribute(obj, 'instance_value')),
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
            gorilla.Patch(destination, 'parent_value', gorilla.get_attribute(obj, 'parent_value')),
            gorilla.Patch(destination, 'to_value', gorilla.get_attribute(obj, 'to_value')),
        ]
        self.assertEqual(patches, expected_patches)

    def test_create_patches_3(self):
        def filter(name, value):
            return 'method' in name

        destination = _tomodule
        obj = _frommodule
        patches = gorilla.create_patches(destination, obj, filter=filter,
                                         use_decorators=False)
        expected_patches = [
            gorilla.Patch(destination, 'unbound_class_method', gorilla.get_attribute(obj, 'unbound_class_method')),
            gorilla.Patch(destination, 'unbound_method', gorilla.get_attribute(obj, 'unbound_method')),
            gorilla.Patch(destination, 'unbound_static_method', gorilla.get_attribute(obj, 'unbound_static_method')),
        ]
        self.assertEqual(patches, expected_patches)

        destination = _tomodule.Class
        obj = _frommodule.Class
        patches = gorilla.create_patches(destination, obj, filter=filter,
                                         use_decorators=False)
        expected_patches = [
            gorilla.Patch(destination, 'class_method', gorilla.get_attribute(obj, 'class_method')),
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
            gorilla.Patch(destination, 'static_method', gorilla.get_attribute(obj, 'static_method')),
        ]
        self.assertEqual(patches, expected_patches)

        destination = _tomodule.Parent
        obj = _frommodule.Parent
        patches = gorilla.create_patches(destination, obj, filter=filter,
                                         use_decorators=False)
        expected_patches = [
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
        ]
        self.assertEqual(patches, expected_patches)

        destination = _tomodule.Child
        obj = _frommodule.Child
        patches = gorilla.create_patches(destination, obj, filter=filter,
                                         use_decorators=False)
        expected_patches = [
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
        ]
        self.assertEqual(patches, expected_patches)

    def test_create_patches_4(self):
        def filter(name, value):
            return 'method' in name

        destination = _tomodule
        obj = _frommodule
        patches = gorilla.create_patches(destination, obj, filter=filter)
        expected_patches = [
            gorilla.Patch(destination, 'function', gorilla.get_attribute(obj, 'function')),
            gorilla.Patch(destination, 'whatever', gorilla.get_attribute(obj, 'unbound_class_method')),
            gorilla.Patch(destination, 'unbound_static_method', gorilla.get_attribute(obj, 'unbound_static_method'), settings=gorilla.Settings(allow_hit=True))
        ]
        self.assertEqual(patches, expected_patches)

        destination = _tomodule.Class
        obj = _frommodule.Class
        patches = gorilla.create_patches(destination, obj, filter=filter)
        expected_patches = [
            gorilla.Patch(destination, 'class_method', gorilla.get_attribute(obj, 'class_method')),
            gorilla.Patch(destination, 'whatever', gorilla.get_attribute(obj, 'method')),
        ]
        self.assertEqual(patches, expected_patches)

        destination = _tomodule.Parent
        obj = _frommodule.Parent
        patches = gorilla.create_patches(destination, obj, filter=filter)
        expected_patches = [
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
        ]
        self.assertEqual(patches, expected_patches)

        destination = _tomodule.Child
        obj = _frommodule.Child
        patches = gorilla.create_patches(destination, obj, filter=filter)
        expected_patches = [
            gorilla.Patch(destination, 'method', gorilla.get_attribute(obj, 'method')),
        ]
        self.assertEqual(patches, expected_patches)

    def test_create_patches_5(self):
        destination = _tomodule
        obj = _frommodule

        gorilla.name('function')(gorilla.get_attribute(obj, 'Class'))
        gorilla.name('dummy_1')(gorilla.get_attribute(obj, 'Parent'))
        gorilla.name('dummy_2')(gorilla.get_attribute(obj, 'Child'))
        patches = gorilla.create_patches(destination, obj)

        expected_patches = [
            gorilla.Patch(destination, 'dummy_2', gorilla.get_attribute(obj, 'Child')),
            gorilla.Patch(destination, 'function', gorilla.get_attribute(obj, 'Class')),
            gorilla.Patch(destination, 'dummy_1', gorilla.get_attribute(obj, 'Parent')),
            gorilla.Patch(destination, 'function', gorilla.get_attribute(obj, 'function')),
            gorilla.Patch(destination, 'global_variable', gorilla.get_attribute(obj, 'global_variable')),
            gorilla.Patch(destination, 'whatever', gorilla.get_attribute(obj, 'unbound_class_method')),
            gorilla.Patch(destination, 'unbound_static_method', gorilla.get_attribute(obj, 'unbound_static_method'), settings=gorilla.Settings(allow_hit=True)),
        ]
        self.assertEqual(patches, expected_patches)

    def test_create_patches_6(self):
        destination = _tomodule.Class
        obj = _frommodule.Class
        patches = gorilla.create_patches(destination, obj, filter=None,
                                         recursive=False, use_decorators=False)

        expected_patches = [
            gorilla.Patch(destination, name, value)
            for name, value in sorted(_iteritems(obj.__dict__))]

        self.assertEqual(patches, expected_patches)

    def test_find_patches_1(self):
        patches = gorilla.find_patches([_utils])
        expected_patches = [
            gorilla.Patch(_tomodule.Class, 'STATIC_VALUE', gorilla.get_attribute(_frommodule.Class, 'STATIC_VALUE')),
            gorilla.Patch(_tomodule.Class, 'class_method', gorilla.get_attribute(_frommodule.Class, 'class_method')),
            gorilla.Patch(_tomodule.Class, 'whatever', gorilla.get_attribute(_frommodule.Class, 'method')),
            gorilla.Patch(_tomodule.Parent, 'value', gorilla.get_attribute(_frommodule.Class, 'value')),
            gorilla.Patch(_tomodule.Class.Inner, 'STATIC_VALUE', gorilla.get_attribute(_frommodule.Class.Inner, 'STATIC_VALUE')),
            gorilla.Patch(_tomodule.Class.Inner, 'method', gorilla.get_attribute(_frommodule.Class.Inner, 'method')),
            gorilla.Patch(_tomodule, 'function', gorilla.get_attribute(_frommodule, 'function')),
            gorilla.Patch(_tomodule.Parent, 'method', gorilla.get_attribute(_frommodule.Parent, 'method')),
            gorilla.Patch(_tomodule.Parent, 'method', gorilla.get_attribute(_frommodule.Parent, 'method')),
            gorilla.Patch(_tomodule, 'function0', gorilla.get_attribute(_subpackage, 'function')),
            gorilla.Patch(_tomodule, 'function1', gorilla.get_attribute(_module1, 'function')),
            gorilla.Patch(_tomodule.Class, 'unbound_class_method', gorilla.get_attribute(_module1, 'unbound_class_method')),
            gorilla.Patch(_tomodule.Class, 'unbound_method', gorilla.get_attribute(_module1, 'unbound_method')),
            gorilla.Patch(_tomodule.Class, 'unbound_static_method', gorilla.get_attribute(_module1, 'unbound_static_method')),
            gorilla.Patch(_tomodule.Class, 'method1', gorilla.get_attribute(_module1.Class, 'method')),
            gorilla.Patch(_tomodule.Class, 'value1', gorilla.get_attribute(_module1.Class, 'value')),
            gorilla.Patch(_tomodule.Class, 'class_method2', gorilla.get_attribute(_module2.Class, 'class_method')),
            gorilla.Patch(_tomodule.Class, 'static_method2', gorilla.get_attribute(_module2.Class, 'static_method')),
        ]
        self.assertEqual(patches, expected_patches)

        patches = gorilla.find_patches([_utils], recursive=False)
        expected_patches = [
            gorilla.Patch(_tomodule.Class, 'STATIC_VALUE', gorilla.get_attribute(_frommodule.Class, 'STATIC_VALUE')),
            gorilla.Patch(_tomodule.Class, 'class_method', gorilla.get_attribute(_frommodule.Class, 'class_method')),
            gorilla.Patch(_tomodule.Class, 'whatever', gorilla.get_attribute(_frommodule.Class, 'method')),
            gorilla.Patch(_tomodule.Parent, 'value', gorilla.get_attribute(_frommodule.Class, 'value')),
            gorilla.Patch(_tomodule.Class.Inner, 'STATIC_VALUE', gorilla.get_attribute(_frommodule.Class.Inner, 'STATIC_VALUE')),
            gorilla.Patch(_tomodule.Class.Inner, 'method', gorilla.get_attribute(_frommodule.Class.Inner, 'method')),
            gorilla.Patch(_tomodule, 'function', gorilla.get_attribute(_frommodule, 'function')),
            gorilla.Patch(_tomodule.Parent, 'method', gorilla.get_attribute(_frommodule.Parent, 'method')),
            gorilla.Patch(_tomodule.Parent, 'method', gorilla.get_attribute(_frommodule.Parent, 'method')),
        ]
        self.assertEqual(patches, expected_patches)

    def test_find_patches_2(self):
        global _utils
        self.tearDown()
        _utils = importlib.import_module(_utils.__name__)

        patches = gorilla.find_patches([_utils])
        self.setUp()

        expected_patches = [
            gorilla.Patch(_tomodule.Class, 'STATIC_VALUE', gorilla.get_attribute(_frommodule.Class, 'STATIC_VALUE')),
            gorilla.Patch(_tomodule.Class, 'class_method', gorilla.get_attribute(_frommodule.Class, 'class_method')),
            gorilla.Patch(_tomodule.Class, 'whatever', gorilla.get_attribute(_frommodule.Class, 'method')),
            gorilla.Patch(_tomodule.Parent, 'value', gorilla.get_attribute(_frommodule.Class, 'value')),
            gorilla.Patch(_tomodule.Class.Inner, 'STATIC_VALUE', gorilla.get_attribute(_frommodule.Class.Inner, 'STATIC_VALUE')),
            gorilla.Patch(_tomodule.Class.Inner, 'method', gorilla.get_attribute(_frommodule.Class.Inner, 'method')),
            gorilla.Patch(_tomodule, 'function', gorilla.get_attribute(_frommodule, 'function')),
            gorilla.Patch(_tomodule.Parent, 'method', gorilla.get_attribute(_frommodule.Parent, 'method')),
            gorilla.Patch(_tomodule.Parent, 'method', gorilla.get_attribute(_frommodule.Parent, 'method')),
            gorilla.Patch(_tomodule, 'function0', gorilla.get_attribute(_subpackage, 'function')),
            gorilla.Patch(_tomodule, 'function1', gorilla.get_attribute(_module1, 'function')),
            gorilla.Patch(_tomodule.Class, 'unbound_class_method', gorilla.get_attribute(_module1, 'unbound_class_method')),
            gorilla.Patch(_tomodule.Class, 'unbound_method', gorilla.get_attribute(_module1, 'unbound_method')),
            gorilla.Patch(_tomodule.Class, 'unbound_static_method', gorilla.get_attribute(_module1, 'unbound_static_method')),
            gorilla.Patch(_tomodule.Class, 'method1', gorilla.get_attribute(_module1.Class, 'method')),
            gorilla.Patch(_tomodule.Class, 'value1', gorilla.get_attribute(_module1.Class, 'value')),
            gorilla.Patch(_tomodule.Class, 'class_method2', gorilla.get_attribute(_module2.Class, 'class_method')),
            gorilla.Patch(_tomodule.Class, 'static_method2', gorilla.get_attribute(_module2.Class, 'static_method')),
        ]
        self.assertEqual(patches, expected_patches)

        patches = gorilla.find_patches([_utils], recursive=False)
        expected_patches = [
            gorilla.Patch(_tomodule.Class, 'STATIC_VALUE', gorilla.get_attribute(_frommodule.Class, 'STATIC_VALUE')),
            gorilla.Patch(_tomodule.Class, 'class_method', gorilla.get_attribute(_frommodule.Class, 'class_method')),
            gorilla.Patch(_tomodule.Class, 'whatever', gorilla.get_attribute(_frommodule.Class, 'method')),
            gorilla.Patch(_tomodule.Parent, 'value', gorilla.get_attribute(_frommodule.Class, 'value')),
            gorilla.Patch(_tomodule.Class.Inner, 'STATIC_VALUE', gorilla.get_attribute(_frommodule.Class.Inner, 'STATIC_VALUE')),
            gorilla.Patch(_tomodule.Class.Inner, 'method', gorilla.get_attribute(_frommodule.Class.Inner, 'method')),
            gorilla.Patch(_tomodule, 'function', gorilla.get_attribute(_frommodule, 'function')),
            gorilla.Patch(_tomodule.Parent, 'method', gorilla.get_attribute(_frommodule.Parent, 'method')),
            gorilla.Patch(_tomodule.Parent, 'method', gorilla.get_attribute(_frommodule.Parent, 'method')),
        ]
        self.assertEqual(patches, expected_patches)

    def test_get_attribute(self):
        self.assertIs(gorilla.get_attribute(_frommodule.Class, 'STATIC_VALUE'), _frommodule.Class.__dict__['STATIC_VALUE'])
        self.assertIs(gorilla.get_attribute(_frommodule.Class, '__init__'), _frommodule.Class.__dict__['__init__'])
        self.assertIs(gorilla.get_attribute(_frommodule.Class, 'value'), _frommodule.Class.__dict__['value'])
        self.assertIs(gorilla.get_attribute(_frommodule.Class, 'method'), _frommodule.Class.__dict__['method'])
        self.assertIs(gorilla.get_attribute(_frommodule.Class, 'class_method'), _frommodule.Class.__dict__['class_method'])
        self.assertIs(gorilla.get_attribute(_frommodule.Class, 'static_method'), _frommodule.Class.__dict__['static_method'])
        self.assertIs(gorilla.get_attribute(_frommodule.Parent, 'STATIC_VALUE'), _frommodule.Parent.__dict__['STATIC_VALUE'])
        self.assertIs(gorilla.get_attribute(_frommodule.Parent, '__init__'), _frommodule.Parent.__dict__['__init__'])
        self.assertIs(gorilla.get_attribute(_frommodule.Parent, 'method'), _frommodule.Parent.__dict__['method'])
        self.assertIs(gorilla.get_attribute(_frommodule.Child, 'STATIC_VALUE'), _frommodule.Parent.__dict__['STATIC_VALUE'])
        self.assertIs(gorilla.get_attribute(_frommodule.Child, '__init__'), _frommodule.Child.__dict__['__init__'])
        self.assertIs(gorilla.get_attribute(_frommodule.Child, 'method'), _frommodule.Parent.__dict__['method'])

    def test_get_original_attribute(self):
        destination = _tomodule.Class
        name = 'method'
        target = gorilla.get_attribute(destination, name)
        obj = gorilla.get_attribute(_frommodule, 'unbound_method')
        settings = gorilla.Settings(allow_hit=True)
        patch = gorilla.Patch(destination, name, obj, settings=settings)

        gorilla.apply(patch)
        self.assertIs(_unfold(gorilla.get_original_attribute(destination, name)), target)

    def test__get_members_1(self):
        members = gorilla._get_members(_frommodule)
        expected_members = [
            ('Child', gorilla.get_attribute(_frommodule, 'Child')),
            ('Class', gorilla.get_attribute(_frommodule, 'Class')),
            ('Parent', gorilla.get_attribute(_frommodule, 'Parent')),
            ('function', gorilla.get_attribute(_frommodule, 'function')),
            ('global_variable', gorilla.get_attribute(_frommodule, 'global_variable')),
            ('unbound_class_method', gorilla.get_attribute(_frommodule, 'unbound_class_method')),
            ('unbound_method', gorilla.get_attribute(_frommodule, 'unbound_method')),
            ('unbound_static_method', gorilla.get_attribute(_frommodule, 'unbound_static_method')),
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Parent, 'STATIC_VALUE')),
            ('child_value', gorilla.get_attribute(_frommodule.Child, 'child_value')),
            ('from_value', gorilla.get_attribute(_frommodule.Parent, 'from_value')),
            ('instance_value', gorilla.get_attribute(_frommodule.Parent, 'instance_value')),
            ('method', gorilla.get_attribute(_frommodule.Parent, 'method')),
            ('parent_value', gorilla.get_attribute(_frommodule.Parent, 'parent_value')),
            ('to_value', gorilla.get_attribute(_frommodule.Parent, 'to_value')),
            ('Inner', _frommodule.Class.Inner),
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Class, 'STATIC_VALUE')),
            ('class_method', gorilla.get_attribute(_frommodule.Class, 'class_method')),
            ('method', gorilla.get_attribute(_frommodule.Class, 'method')),
            ('static_method', gorilla.get_attribute(_frommodule.Class, 'static_method')),
            ('value', gorilla.get_attribute(_frommodule.Class, 'value')),
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Parent, 'STATIC_VALUE')),
            ('from_value', gorilla.get_attribute(_frommodule.Parent, 'from_value')),
            ('instance_value', gorilla.get_attribute(_frommodule.Parent, 'instance_value')),
            ('method', gorilla.get_attribute(_frommodule.Parent, 'method')),
            ('parent_value', gorilla.get_attribute(_frommodule.Parent, 'parent_value')),
            ('to_value', gorilla.get_attribute(_frommodule.Parent, 'to_value')),
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Class.Inner, 'STATIC_VALUE')),
            ('method', gorilla.get_attribute(_frommodule.Class.Inner, 'method')),
        ]
        self.assertEqual(members, expected_members)

        members = gorilla._get_members(_frommodule.Class)
        expected_members = [
            ('Inner', _frommodule.Class.Inner),
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Class, 'STATIC_VALUE')),
            ('class_method', gorilla.get_attribute(_frommodule.Class, 'class_method')),
            ('method', gorilla.get_attribute(_frommodule.Class, 'method')),
            ('static_method', gorilla.get_attribute(_frommodule.Class, 'static_method')),
            ('value', gorilla.get_attribute(_frommodule.Class, 'value')),
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Class.Inner, 'STATIC_VALUE')),
            ('method', gorilla.get_attribute(_frommodule.Class.Inner, 'method')),
        ]
        self.assertEqual(members, expected_members)

        members = gorilla._get_members(_frommodule.Parent)
        expected_members = [
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Parent, 'STATIC_VALUE')),
            ('from_value', gorilla.get_attribute(_frommodule.Parent, 'from_value')),
            ('instance_value', gorilla.get_attribute(_frommodule.Parent, 'instance_value')),
            ('method', gorilla.get_attribute(_frommodule.Parent, 'method')),
            ('parent_value', gorilla.get_attribute(_frommodule.Parent, 'parent_value')),
            ('to_value', gorilla.get_attribute(_frommodule.Parent, 'to_value')),
        ]
        self.assertEqual(members, expected_members)

        members = gorilla._get_members(_frommodule.Child)
        expected_members = [
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Parent, 'STATIC_VALUE')),
            ('child_value', gorilla.get_attribute(_frommodule.Child, 'child_value')),
            ('from_value', gorilla.get_attribute(_frommodule.Parent, 'from_value')),
            ('instance_value', gorilla.get_attribute(_frommodule.Parent, 'instance_value')),
            ('method', gorilla.get_attribute(_frommodule.Parent, 'method')),
            ('parent_value', gorilla.get_attribute(_frommodule.Parent, 'parent_value')),
            ('to_value', gorilla.get_attribute(_frommodule.Parent, 'to_value')),
        ]
        self.assertEqual(members, expected_members)

    def test__get_members_2(self):
        members = gorilla._get_members(_frommodule, traverse_bases=False)
        expected_members = [
            ('Child', gorilla.get_attribute(_frommodule, 'Child')),
            ('Class', gorilla.get_attribute(_frommodule, 'Class')),
            ('Parent', gorilla.get_attribute(_frommodule, 'Parent')),
            ('function', gorilla.get_attribute(_frommodule, 'function')),
            ('global_variable', gorilla.get_attribute(_frommodule, 'global_variable')),
            ('unbound_class_method', gorilla.get_attribute(_frommodule, 'unbound_class_method')),
            ('unbound_method', gorilla.get_attribute(_frommodule, 'unbound_method')),
            ('unbound_static_method', gorilla.get_attribute(_frommodule, 'unbound_static_method')),
            ('child_value', gorilla.get_attribute(_frommodule.Child, 'child_value')),
            ('Inner', _frommodule.Class.Inner),
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Class, 'STATIC_VALUE')),
            ('class_method', gorilla.get_attribute(_frommodule.Class, 'class_method')),
            ('method', gorilla.get_attribute(_frommodule.Class, 'method')),
            ('static_method', gorilla.get_attribute(_frommodule.Class, 'static_method')),
            ('value', gorilla.get_attribute(_frommodule.Class, 'value')),
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Parent, 'STATIC_VALUE')),
            ('from_value', gorilla.get_attribute(_frommodule.Parent, 'from_value')),
            ('instance_value', gorilla.get_attribute(_frommodule.Parent, 'instance_value')),
            ('method', gorilla.get_attribute(_frommodule.Parent, 'method')),
            ('parent_value', gorilla.get_attribute(_frommodule.Parent, 'parent_value')),
            ('to_value', gorilla.get_attribute(_frommodule.Parent, 'to_value')),
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Class.Inner, 'STATIC_VALUE')),
            ('method', gorilla.get_attribute(_frommodule.Class.Inner, 'method')),
        ]
        self.assertEqual(members, expected_members)

        members = gorilla._get_members(_frommodule.Class, traverse_bases=False)
        expected_members = [
            ('Inner', _frommodule.Class.Inner),
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Class, 'STATIC_VALUE')),
            ('class_method', gorilla.get_attribute(_frommodule.Class, 'class_method')),
            ('method', gorilla.get_attribute(_frommodule.Class, 'method')),
            ('static_method', gorilla.get_attribute(_frommodule.Class, 'static_method')),
            ('value', gorilla.get_attribute(_frommodule.Class, 'value')),
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Class.Inner, 'STATIC_VALUE')),
            ('method', gorilla.get_attribute(_frommodule.Class.Inner, 'method')),
        ]
        self.assertEqual(members, expected_members)

        members = gorilla._get_members(_frommodule.Parent, traverse_bases=False)
        expected_members = [
            ('STATIC_VALUE', gorilla.get_attribute(_frommodule.Parent, 'STATIC_VALUE')),
            ('from_value', gorilla.get_attribute(_frommodule.Parent, 'from_value')),
            ('instance_value', gorilla.get_attribute(_frommodule.Parent, 'instance_value')),
            ('method', gorilla.get_attribute(_frommodule.Parent, 'method')),
            ('parent_value', gorilla.get_attribute(_frommodule.Parent, 'parent_value')),
            ('to_value', gorilla.get_attribute(_frommodule.Parent, 'to_value')),
        ]
        self.assertEqual(members, expected_members)

        members = gorilla._get_members(_frommodule.Child, traverse_bases=False)
        expected_members = [
            ('child_value', gorilla.get_attribute(_frommodule.Child, 'child_value')),
        ]
        self.assertEqual(members, expected_members)

    def test__get_members_3(self):
        obj = _frommodule
        members = gorilla._get_members(obj, recursive=False)
        expected_members = [
            ('Child', gorilla.get_attribute(obj, 'Child')),
            ('Class', gorilla.get_attribute(obj, 'Class')),
            ('Parent', gorilla.get_attribute(obj, 'Parent')),
            ('function', gorilla.get_attribute(obj, 'function')),
            ('global_variable', gorilla.get_attribute(obj, 'global_variable')),
            ('unbound_class_method', gorilla.get_attribute(obj, 'unbound_class_method')),
            ('unbound_method', gorilla.get_attribute(obj, 'unbound_method')),
            ('unbound_static_method', gorilla.get_attribute(obj, 'unbound_static_method')),
        ]
        self.assertEqual(members, expected_members)

        obj = _frommodule.Class
        members = gorilla._get_members(obj, recursive=False)
        expected_members = [
            ('Inner', gorilla.get_attribute(obj, 'Inner')),
            ('STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            ('class_method', gorilla.get_attribute(obj, 'class_method')),
            ('method', gorilla.get_attribute(obj, 'method')),
            ('static_method', gorilla.get_attribute(obj, 'static_method')),
            ('value', gorilla.get_attribute(obj, 'value')),
        ]
        self.assertEqual(members, expected_members)

        obj = _frommodule.Parent
        members = gorilla._get_members(obj, recursive=False)
        expected_members = [
            ('STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            ('from_value', gorilla.get_attribute(obj, 'from_value')),
            ('instance_value', gorilla.get_attribute(obj, 'instance_value')),
            ('method', gorilla.get_attribute(obj, 'method')),
            ('parent_value', gorilla.get_attribute(obj, 'parent_value')),
            ('to_value', gorilla.get_attribute(obj, 'to_value')),
        ]
        self.assertEqual(members, expected_members)

        obj = _frommodule.Child
        members = gorilla._get_members(obj, recursive=False)
        expected_members = [
            ('STATIC_VALUE', gorilla.get_attribute(obj, 'STATIC_VALUE')),
            ('child_value', gorilla.get_attribute(obj, 'child_value')),
            ('from_value', gorilla.get_attribute(obj, 'from_value')),
            ('instance_value', gorilla.get_attribute(obj, 'instance_value')),
            ('method', gorilla.get_attribute(obj, 'method')),
            ('parent_value', gorilla.get_attribute(obj, 'parent_value')),
            ('to_value', gorilla.get_attribute(obj, 'to_value')),
        ]
        self.assertEqual(members, expected_members)


if __name__ == '__main__':
    from tests.run import run
    run('__main__')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

import collections
import importlib
import itertools
import os
import sys

_HERE = os.path.abspath(os.path.dirname(__file__))
sys.path.insert(0, os.path.abspath(os.path.join(_HERE, os.pardir)))

import gorilla

import tests.core as _core
import tests.core.frommodule as _frommodule
import tests.core.tomodule as _tomodule
from tests._testcase import GorillaTestCase


_MODULES = [
    ('_core', _core.__name__),
    ('_frommodule', _frommodule.__name__),
    ('_tomodule', _tomodule.__name__),
]

_CLS_REFERENCES = {
    'Class': 'Class',
    'Class.Inner': 'Class.Inner',
    'Parent': 'Parent',
    'Child': 'Parent',
}


def _list_attribute_paths(obj):
    attributes = []
    stack = collections.deque(((obj, []),))
    while stack:
        obj, parent_path = stack.popleft()
        for attribute in getattr(obj, '__all__', []):
            path = parent_path + [attribute]
            stack.append((getattr(obj, attribute), path))
            attributes.append('.'.join(path))

    return attributes


def _get_attribute_from_path(module, path):
    if path == '':
        return module

    obj = module
    for part in path.split('.'):
        obj = gorilla.get_attribute(obj, part)

    return obj


def _split_attribute_path(path):
    parts = path.split('.')
    return ('.'.join(parts[:-1]), parts[-1])


class CoreTest(GorillaTestCase):

    def setUp(self):
        for module, path in _MODULES:
            globals()[module] = importlib.import_module(path)

    def tearDown(self):
        for module, path in _MODULES:
            if path in sys.modules:
                del sys.modules[path]

    def test_settings(self):
        settings_1 = gorilla.Settings()
        settings_2 = gorilla.Settings(allow_hit=False, store_hit=True)
        self.assertEqual(settings_1, settings_2)
        self.assertNotEqual(settings_1, {'allow_hit': False, 'store_hit': True})

        settings_1.allow_hit = True
        self.assertNotEqual(settings_1, settings_2)

        settings_2.allow_hit = True
        self.assertEqual(settings_1, settings_2)

        settings_1.some_value = 123
        self.assertNotEqual(settings_1, settings_2)

        settings_2.some_value = 123
        self.assertEqual(settings_1, settings_2)

    def test_patch(self):
        patch_1 = gorilla.Patch(_tomodule, 'dummy', _frommodule.function)
        patch_2 = gorilla.Patch(_tomodule, 'dummy', _frommodule.function, settings=None)
        self.assertEqual(patch_1, patch_2)
        self.assertNotEqual(patch_1, {'destination': _tomodule, 'name': 'dummy', 'obj': _frommodule.function, 'settings': None})

        patch_1.name = 'function'
        self.assertNotEqual(patch_1, patch_2)

        patch_2.name = 'function'
        self.assertEqual(patch_1, patch_2)

        patch_1.some_value = 123
        self.assertNotEqual(patch_1, patch_2)

        patch_2.some_value = 123
        self.assertEqual(patch_1, patch_2)

    def test_apply_patch_no_hit(self):
        name = 'dummy'
        settings = gorilla.Settings()

        source_paths = [''] + _list_attribute_paths(_frommodule)
        target_paths = _list_attribute_paths(_tomodule)

        branch_count = 0

        # Retrieve the destinations in two passes instead of directly using a
        # set in order to preserve the ordering.
        seen = set()
        destination_paths = [_split_attribute_path(path)[0]
                             for path in target_paths]
        destination_paths = [path for path in destination_paths
                             if path not in seen and seen.add(path) is None]
        combinations = itertools.product(source_paths, destination_paths)
        for source_path, destination_path in combinations:
            self.setUp()

            destination = _get_attribute_from_path(_tomodule, destination_path)
            obj = _get_attribute_from_path(_frommodule, source_path)
            patch = gorilla.Patch(destination, name, obj, settings=settings)
            gorilla.apply(patch)
            self.assertIs(destination, _get_attribute_from_path(_tomodule, destination_path))

            result = gorilla.get_attribute(destination, name)
            self.assertIs(result, obj)

            if source_path == '':
                branch_count += 1
                self.assertEqual(result.global_variable, "frommodule.global_variable")
                self.assertEqual(result.function(), "frommodule.function (frommodule.Class.STATIC_VALUE)")
                self.assertEqual(result.Class.STATIC_VALUE, "frommodule.Class.STATIC_VALUE")
                self.assertEqual(result.Class.Inner.STATIC_VALUE, "frommodule.Class.Inner.STATIC_VALUE")
                self.assertEqual(result.Parent.STATIC_VALUE, "frommodule.Parent.STATIC_VALUE")
                self.assertEqual(result.Child.STATIC_VALUE, "frommodule.Parent.STATIC_VALUE")
            elif source_path == 'global_variable':
                branch_count += 1
                self.assertEqual(result, "frommodule.global_variable")
            elif source_path == 'function':
                branch_count += 1
                self.assertEqual(result(), "frommodule.function (frommodule.Class.STATIC_VALUE)")
            elif source_path == 'unbound_method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES:
                    branch_count += 1
                    self.assertEqual(getattr(destination(), name)(), "frommodule.unbound_method (tomodule.%s.STATIC_VALUE, tomodule.%s.instance_value)" % ((_CLS_REFERENCES[destination_path],) * 2))
            elif source_path == 'unbound_class_method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES:
                    branch_count += 1
                    self.assertEqual(getattr(destination, name)(), "frommodule.unbound_class_method (tomodule.%s.STATIC_VALUE)" % (_CLS_REFERENCES[destination_path],))
            elif source_path == 'unbound_static_method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES:
                    branch_count += 1
                    self.assertEqual(getattr(destination, name)(), "frommodule.unbound_static_method (frommodule.Class.STATIC_VALUE)")
            elif source_path == 'Class':
                branch_count += 1
                self.assertEqual(result.STATIC_VALUE, "frommodule.Class.STATIC_VALUE")
            elif source_path == 'Class.Inner':
                branch_count += 1
                self.assertEqual(result.STATIC_VALUE, "frommodule.Class.Inner.STATIC_VALUE")
            elif source_path == 'Class.value':
                branch_count += 1
                if destination_path in _CLS_REFERENCES:
                    branch_count += 1
                    instance = destination()
                    self.assertEqual(getattr(instance, name), "frommodule.Class.value.getter (tomodule.%s.instance_value)"  % (_CLS_REFERENCES[destination_path],))
                    setattr(instance, name, 'hello')
                    self.assertEqual(getattr(instance, name), "frommodule.Class.value.getter (hello)")
            elif source_path == 'Class.method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES:
                    branch_count += 1
                    self.assertEqual(getattr(destination(), name)(), "frommodule.Class.method (tomodule.%s.STATIC_VALUE, tomodule.%s.instance_value)" % ((_CLS_REFERENCES[destination_path],) * 2))
            elif source_path == 'Class.class_method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES:
                    branch_count += 1
                    self.assertEqual(getattr(destination, name)(), "frommodule.Class.class_method (tomodule.%s.STATIC_VALUE)" % (_CLS_REFERENCES[destination_path],))
            elif source_path == 'Class.static_method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES:
                    branch_count += 1
                    self.assertEqual(getattr(destination, name)(), "frommodule.Class.static_method (frommodule.Class.STATIC_VALUE)")
            elif source_path == 'Parent':
                branch_count += 1
                self.assertEqual(result.__slots__, ('instance_value', 'parent_value', 'to_value', 'from_value'))
                self.assertEqual(result().parent_value, "frommodule.Parent.parent_value")
                self.assertEqual(result().method(), "frommodule.Parent.method (frommodule.Parent.instance_value)")
            elif source_path == 'Parent.method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES:
                    branch_count += 1
                    self.assertEqual(getattr(destination(), name)(), "frommodule.Parent.method (tomodule.%s.instance_value)" % (_CLS_REFERENCES[destination_path],))
            elif source_path == 'Child':
                branch_count += 1
                self.assertEqual(result.__slots__, ('child_value',))
                self.assertEqual(result().parent_value, "frommodule.Parent.parent_value")
                self.assertEqual(result().child_value, "frommodule.Child.child_value")
                self.assertEqual(result().method(), "frommodule.Parent.method (frommodule.Parent.instance_value)")
            elif source_path == 'Child.method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES:
                    branch_count += 1
                    self.assertEqual(getattr(destination(), name)(), "frommodule.Parent.method (tomodule.%s.instance_value)" % (_CLS_REFERENCES[destination_path],))

            self.tearDown()

        # Make sure that all test branches are covered.
        self.assertEqual(branch_count, 116)

    def test_apply_patch_with_hit_1(self):
        settings = gorilla.Settings()

        source_paths = [''] + _list_attribute_paths(_frommodule)
        target_paths = _list_attribute_paths(_tomodule)
        combinations = itertools.product(source_paths, target_paths)
        for source_path, target_path in combinations:
            self.setUp()

            destination_path, name = _split_attribute_path(target_path)
            destination = _get_attribute_from_path(_tomodule, destination_path)
            obj = _get_attribute_from_path(_frommodule, source_path)
            patch = gorilla.Patch(destination, name, obj, settings=settings)
            self.assertRaises(RuntimeError, gorilla.apply, patch)

            self.tearDown()

    def test_apply_patch_with_hit_2(self):
        settings = gorilla.Settings(allow_hit=True)

        branch_count = 0

        source_paths = [''] + _list_attribute_paths(_frommodule)
        target_paths = _list_attribute_paths(_tomodule)
        combinations = itertools.product(source_paths, target_paths)
        for source_path, target_path in combinations:
            self.setUp()

            destination_path, name = _split_attribute_path(target_path)
            destination = _get_attribute_from_path(_tomodule, destination_path)
            target = gorilla.get_attribute(destination, name)
            obj = _get_attribute_from_path(_frommodule, source_path)
            patch = gorilla.Patch(destination, name, obj, settings=settings)
            gorilla.apply(patch)
            self.assertIs(destination, _get_attribute_from_path(_tomodule, destination_path))

            result = gorilla.get_attribute(destination, name)
            self.assertIs(result, obj)

            if source_path == '':
                branch_count += 1
                self.assertEqual(result.global_variable, "frommodule.global_variable")
                self.assertEqual(result.function(), "frommodule.function (frommodule.Class.STATIC_VALUE)")
                self.assertEqual(result.Class.STATIC_VALUE, "frommodule.Class.STATIC_VALUE")
                self.assertEqual(result.Class.Inner.STATIC_VALUE, "frommodule.Class.Inner.STATIC_VALUE")
                self.assertEqual(result.Parent.STATIC_VALUE, "frommodule.Parent.STATIC_VALUE")
                self.assertEqual(result.Child.STATIC_VALUE, "frommodule.Parent.STATIC_VALUE")
            elif source_path == 'global_variable':
                branch_count += 1
                self.assertEqual(result, "frommodule.global_variable")
                if destination_path in _CLS_REFERENCES and name in ('STATIC_VALUE',):
                    branch_count += 1
                    self.assertEqual(destination.STATIC_VALUE, "frommodule.global_variable")

                if target_path == 'Class.STATIC_VALUE':
                    branch_count += 1
                    self.assertEqual(destination.class_method(), "tomodule.Class.class_method (frommodule.global_variable)")
                    self.assertEqual(destination.static_method(), "tomodule.Class.static_method (frommodule.global_variable)")
            elif source_path == 'function':
                branch_count += 1
                self.assertEqual(result(), "frommodule.function (frommodule.Class.STATIC_VALUE)")
            elif source_path == 'unbound_method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES and name not in ('STATIC_VALUE', '__init__'):
                    branch_count += 1
                    self.assertEqual(getattr(destination(), name)(), "frommodule.unbound_method (tomodule.%s.STATIC_VALUE, tomodule.%s.instance_value)" % ((_CLS_REFERENCES[destination_path],) * 2))
            elif source_path == 'unbound_class_method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES and name not in ('STATIC_VALUE',):
                    branch_count += 1
                    self.assertEqual(getattr(destination, name)(), "frommodule.unbound_class_method (tomodule.%s.STATIC_VALUE)" % (_CLS_REFERENCES[destination_path],))
            elif source_path == 'unbound_static_method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES:
                    branch_count += 1
                    self.assertEqual(getattr(destination, name)(), "frommodule.unbound_static_method (frommodule.Class.STATIC_VALUE)")
            elif source_path == 'Class':
                branch_count += 1
                self.assertEqual(result.STATIC_VALUE, "frommodule.Class.STATIC_VALUE")
            elif source_path == 'Class.Inner':
                branch_count += 1
                self.assertEqual(result.STATIC_VALUE, "frommodule.Class.Inner.STATIC_VALUE")
            elif source_path == 'Class.value':
                branch_count += 1
                if destination_path in _CLS_REFERENCES and name not in ('__init__',):
                    branch_count += 1
                    instance = destination()
                    self.assertEqual(getattr(instance, name), "frommodule.Class.value.getter (tomodule.%s.instance_value)"  % (_CLS_REFERENCES[destination_path],))
                    setattr(instance, name, 'hello')
                    self.assertEqual(getattr(instance, name), "frommodule.Class.value.getter (hello)")
            elif source_path == 'Class.method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES and name not in ('STATIC_VALUE', '__init__'):
                    branch_count += 1
                    self.assertEqual(getattr(destination(), name)(), "frommodule.Class.method (tomodule.%s.STATIC_VALUE, tomodule.%s.instance_value)" % ((_CLS_REFERENCES[destination_path],) * 2))
            elif source_path == 'Class.class_method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES and name not in ('STATIC_VALUE',):
                    branch_count += 1
                    self.assertEqual(getattr(destination, name)(), "frommodule.Class.class_method (tomodule.%s.STATIC_VALUE)" % (_CLS_REFERENCES[destination_path],))
            elif source_path == 'Class.static_method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES:
                    branch_count += 1
                    self.assertEqual(getattr(destination, name)(), "frommodule.Class.static_method (frommodule.Class.STATIC_VALUE)")
            elif source_path == 'Parent':
                branch_count += 1
                self.assertEqual(result.__slots__, ('instance_value', 'parent_value', 'to_value', 'from_value'))
                self.assertEqual(result().parent_value, "frommodule.Parent.parent_value")
                self.assertEqual(result().method(), "frommodule.Parent.method (frommodule.Parent.instance_value)")
            elif source_path == 'Parent.method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES and name not in ('__init__',):
                    branch_count += 1
                    self.assertEqual(getattr(destination(), name)(), "frommodule.Parent.method (tomodule.%s.instance_value)" % (_CLS_REFERENCES[destination_path],))
            elif source_path == 'Child':
                branch_count += 1
                self.assertEqual(result.__slots__, ('child_value',))
                self.assertEqual(result().parent_value, "frommodule.Parent.parent_value")
                self.assertEqual(result().child_value, "frommodule.Child.child_value")
                self.assertEqual(result().method(), "frommodule.Parent.method (frommodule.Parent.instance_value)")
            elif source_path == 'Child.method':
                branch_count += 1
                if destination_path in _CLS_REFERENCES and name not in ('__init__',):
                    branch_count += 1
                    self.assertEqual(getattr(destination(), name)(), "frommodule.Parent.method (tomodule.%s.instance_value)" % (_CLS_REFERENCES[destination_path],))

            self.tearDown()

        # Make sure that all test branches are covered.
        self.assertEqual(branch_count, 427)

    def test_apply_patch_with_hit_3(self):
        settings = gorilla.Settings(allow_hit=True, store_hit=False)

        source_paths = [''] + _list_attribute_paths(_frommodule)
        target_paths = _list_attribute_paths(_tomodule)
        combinations = itertools.product(source_paths, target_paths)
        for source_path, target_path in combinations:
            self.setUp()

            destination_path, name = _split_attribute_path(target_path)
            destination = _get_attribute_from_path(_tomodule, destination_path)
            obj = _get_attribute_from_path(_frommodule, source_path)
            patch = gorilla.Patch(destination, name, obj, settings=settings)
            gorilla.apply(patch)
            self.assertIs(destination, _get_attribute_from_path(_tomodule, destination_path))

            result = gorilla.get_attribute(destination, name)
            self.assertIs(result, obj)

            self.assertRaises(AttributeError, gorilla.get_original_attribute, destination, name)

            self.tearDown()

    def test_revert_patch(self):
        self.setUp()

        settings = gorilla.Settings(allow_hit=True)
        destination = _tomodule
        obj = _frommodule.global_variable

        patch = gorilla.Patch(destination, 'dummy', obj, settings=settings)
        gorilla.apply(patch)
        self.assertEqual(_tomodule.dummy, "frommodule.global_variable")
        gorilla.revert(patch)
        self.assertRaises(AttributeError, getattr, _tomodule, 'dummy')


        patch = gorilla.Patch(destination, 'global_variable', obj, settings=settings)
        gorilla.apply(patch)
        self.assertEqual(_tomodule.global_variable, "frommodule.global_variable")
        gorilla.revert(patch)
        self.assertEqual(_tomodule.global_variable, "tomodule.global_variable")

        self.tearDown()

    def test_stack_patches(self):
        self.setUp()

        settings = gorilla.Settings(allow_hit=True)
        destination = _tomodule

        patch = gorilla.Patch(_tomodule, 'stack', _frommodule.stack_1, settings=settings)
        gorilla.apply(patch, id='first')

        patch = gorilla.Patch(_tomodule, 'stack', _frommodule.stack_2, settings=settings)
        gorilla.apply(patch, id='second')

        self.assertEqual(_tomodule.stack(), ("blue", "white", "red"))

        self.tearDown()


if __name__ == '__main__':
    from tests.run import run
    run('__main__')



# -*- coding: utf-8 -*-

__all__ = ['global_variable', 'function', 'Class', 'Parent', 'Child']


#: tomodule.global_variable
global_variable = "tomodule.global_variable"


def function():
    """tomodule.function"""
    return "tomodule.function (%s)" % (Class.STATIC_VALUE,)


class Class(object):

    """tomodule.Class"""

    __all__ = ['STATIC_VALUE', 'Inner', '__init__', 'value', 'method',
               'class_method', 'static_method']

    STATIC_VALUE = "tomodule.Class.STATIC_VALUE"

    class Inner(object):

        """tomodule.Class.Inner"""

        __all__ = ['STATIC_VALUE', '__init__', 'method']

        STATIC_VALUE = "tomodule.Class.Inner.STATIC_VALUE"

        def __init__(self):
            """tomodule.Class.Inner.__init__"""
            self.instance_value = "tomodule.Class.Inner.instance_value"

        def method(self):
            """tomodule.Class.Inner.method"""
            return "tomodule.Class.Inner.method (%s)" % (
                self.instance_value,)


    def __init__(self):
        """tomodule.Class.__init__"""
        self.instance_value = "tomodule.Class.instance_value"

    @property
    def value(self):
        """tomodule.Class.value.getter"""
        return "tomodule.Class.value.getter (%s)" % (self.instance_value,)

    @value.setter
    def value(self, value):
        """tomodule.Class.value.setter"""
        self.instance_value = value

    def method(self):
        """tomodule.Class.method"""
        return "tomodule.Class.method (%s, %s)" % (self.STATIC_VALUE,
                                                   self.instance_value)

    @classmethod
    def class_method(cls):
        """tomodule.Class.class_method"""
        return "tomodule.Class.class_method (%s)" % (cls.STATIC_VALUE,)

    @staticmethod
    def static_method():
        """tomodule.Class.static_method"""
        return "tomodule.Class.static_method (%s)" % (Class.STATIC_VALUE,)


class Parent(object):

    """tomodule.Parent"""

    __all__ = ['STATIC_VALUE', '__init__', 'method']
    __slots__ = ('instance_value', 'parent_value', 'to_value',)

    STATIC_VALUE = "tomodule.Parent.STATIC_VALUE"

    def __init__(self):
        """tomodule.Parent.__init__"""
        self.instance_value = "tomodule.Parent.instance_value"
        self.parent_value = "tomodule.Parent.parent_value"

    def method(self):
        """tomodule.Parent.method"""
        return "tomodule.Parent.method (%s)" % (self.instance_value,)


class Child(Parent):

    """tomodule.Child"""

    __all__ = ['__init__', 'method']
    __slots__ = ('child_value',)

    def __init__(self):
        """tomodule.Child.__init__"""
        super(Child, self).__init__()
        self.child_value = "tomodule.Child.child_value"


def stack():
    return ("blue",)



# -*- coding: utf-8 -*-

import gorilla

import tests.core.tomodule


__all__ = ['global_variable', 'function', 'unbound_method',
           'unbound_class_method', 'unbound_static_method', 'Class', 'Parent',
           'Child']


#: frommodule.global_variable
global_variable = "frommodule.global_variable"


def function():
    """frommodule.function"""
    return "frommodule.function (%s)" % (Class.STATIC_VALUE,)


def unbound_method(self):
    """frommodule.unbound_method"""
    return "frommodule.unbound_method (%s, %s)" % (
        self.STATIC_VALUE, self.instance_value)


@classmethod
def unbound_class_method(cls):
    """frommodule.unbound_class_method"""
    return "frommodule.unbound_class_method (%s)" % (cls.STATIC_VALUE,)


@staticmethod
def unbound_static_method():
    """frommodule.unbound_static_method"""
    return "frommodule.unbound_static_method (%s)" % (Class.STATIC_VALUE,)


class Class(object):

    """frommodule.Class"""

    __all__ = ['STATIC_VALUE', 'Inner', '__init__', 'value', 'method',
               'class_method', 'static_method']

    STATIC_VALUE = "frommodule.Class.STATIC_VALUE"

    class Inner(object):

        """frommodule.Class.Inner"""

        __all__ = ['STATIC_VALUE', '__init__', 'method']

        STATIC_VALUE = "frommodule.Class.Inner.STATIC_VALUE"

        def __init__(self):
            """frommodule.Class.Inner.__init__"""
            self.instance_value = "frommodule.Class.Inner.instance_value"

        def method(self):
            """frommodule.Class.Inner.method"""
            return "frommodule.Class.Inner.method (%s)" % (
                self.instance_value,)


    def __init__(self):
        """frommodule.Class.__init__"""
        gorilla.get_original_attribute(self, '__init__')()
        self.stored_init_instance_value = self.instance_value
        self.instance_value = "frommodule.Class.instance_value"

    @property
    def value(self):
        """frommodule.Class.value.getter"""
        return "frommodule.Class.value.getter (%s)" % (self.instance_value,)

    @value.setter
    def value(self, value):
        """frommodule.Class.value.setter"""
        self.instance_value = value

    def method(self):
        """frommodule.Class.method"""
        return "frommodule.Class.method (%s, %s)" % (self.STATIC_VALUE,
                                                     self.instance_value)

    @classmethod
    def class_method(cls):
        """frommodule.Class.class_method"""
        return "frommodule.Class.class_method (%s)" % (cls.STATIC_VALUE,)

    @staticmethod
    def static_method():
        """frommodule.Class.static_method"""
        return "frommodule.Class.static_method (%s)" % (Class.STATIC_VALUE,)


class Parent(object):

    """frommodule.Parent"""

    __all__ = ['STATIC_VALUE', '__init__', 'method']
    __slots__ = tests.core.tomodule.Parent.__slots__ + ('from_value',)

    STATIC_VALUE = "frommodule.Parent.STATIC_VALUE"

    def __init__(self):
        """frommodule.Parent.__init__"""
        self.instance_value = "frommodule.Parent.instance_value"
        self.parent_value = "frommodule.Parent.parent_value"
        self.to_value = "frommodule.Parent.to_value"
        self.from_value = "frommodule.Parent.from_value"

    def method(self):
        """frommodule.Parent.method"""
        return "frommodule.Parent.method (%s)" % (self.instance_value,)


class Child(Parent):

    """frommodule.Child"""

    __all__ = ['__init__', 'method']
    __slots__ = ('child_value',)

    def __init__(self):
        """frommodule.Child.__init__"""
        super(Child, self).__init__()
        self.child_value = "frommodule.Child.child_value"


def stack_1():
    original = gorilla.get_original_attribute(tests.core.tomodule, 'stack', id='first')()
    return original + ("white",)


def stack_2():
    original = gorilla.get_original_attribute(tests.core.tomodule, 'stack', id='second')()
    return original + ("red",)



# -*- coding: utf-8 -*-

import gorilla

import tests.utils.tomodule


@gorilla.patch(tests.utils.tomodule, name='function0')
def function():
    """subpackage.function"""
    return "subpackage.function"



# -*- coding: utf-8 -*-

import os
import sys
from datetime import datetime

sys.path.insert(0, os.path.abspath(os.pardir))

import gorilla


# -- General configuration ------------------------------------------------

needs_sphinx = '1.3'

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.autosummary',
    'sphinx.ext.doctest',
    'sphinx.ext.napoleon',
    'sphinx.ext.viewcode',
]

templates_path = ['_templates']
source_suffix = '.rst'
master_doc = 'index'

project = gorilla.__title__
version = gorilla.__version__
description = gorilla.__summary__
author = gorilla.__author__
copyright = "2014-%i, %s" % (datetime.utcnow().year, gorilla.__author__)
release = version
language = None

add_module_names = True
autodoc_member_order = 'bysource'
default_role = 'autolink'
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']
pygments_style = 'sphinx'
show_authors = False
todo_include_todos = False


# -- Options for HTML output ----------------------------------------------

html_theme = 'alabaster'
html_theme_options = {
    'description': description,
    'github_user': 'christophercrouzet',
    'github_repo': 'gorilla',
    'github_type': 'star',
    'fixed_sidebar': True,
}
html_sidebars = {
    '**': [
        'about.html',
        'navigation.html',
        'relations.html',
        'links.html',
        'searchbox.html',
        'donate.html',
    ],
}
html_static_path = ['_static']


# -- Options for HTMLHelp output ------------------------------------------

htmlhelp_basename = '%sdoc' % (project,)


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
}

latex_documents = [
    (master_doc, '%s.tex' % (project,), "%s Documentation" % (project,),
     author, 'manual'),
]


# -- Options for manual page output ---------------------------------------

man_pages = [
    (master_doc, project, "%s Documentation" % (project,), [author], 1),
]


# -- Options for Texinfo output -------------------------------------------

texinfo_documents = [
    (master_doc, project, "%s Documentation" % (project,), author, project,
     description, 'Miscellaneous'),
]



# -*- coding: utf-8 -*-

import io
import os
import re
import setuptools


def _read(*paths, **kwargs):
    # Credits: https://packaging.python.org/single_source_version.
    here = os.path.dirname(__file__)
    encoding = kwargs.get('encoding', 'utf8')
    with io.open(os.path.join(here, *paths), encoding=encoding) as f:
        return f.read()


def _get_metas(*file_paths):
    data = _read(*file_paths)
    out = {}
    metas = ('author', 'contact', 'license', 'summary', 'title', 'url',
             'version')
    for meta in metas:
        pattern = r'^__%s__ = u?[\'"]([^\'"]*)[\'"]' % (meta,)
        match = re.search(pattern, data, re.MULTILINE)
        if match is None:
            raise RuntimeError("Unable to find the metadata '%s'." % (meta,))

        out[meta] = match.group(1)

    return out


_METAS = _get_metas('gorilla.py')

setuptools.setup(
    name=_METAS['title'],
    version=_METAS['version'],
    description=_METAS['summary'],
    url=_METAS['url'],
    author=_METAS['author'],
    author_email=_METAS['contact'],
    license=_METAS['license'],
    keywords='gorilla monkey patch patching',
    long_description=_read('README.rst'),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Utilities',
    ],
    extras_require={
        'dev': ['coverage', 'pycodestyle', 'pydocstyle', 'pylint',
                'sphinx>=1.3', 'tox'],
        'docs': ['sphinx>=1.3'],
    },
    packages=[],
    py_modules=['gorilla'],
    include_package_data=True
)



# -*- coding: utf-8 -*-

import gorilla

import tests.utils.tomodule


def function():
    """subpackage.module2.function"""
    return "subpackage.module2.function"


def unbound_method(self):
    """subpackage.module2.unbound_method"""
    return "subpackage.module2.unbound_method"


@classmethod
def unbound_class_method(cls):
    """subpackage.module2.unbound_class_method"""
    return "subpackage.module2.unbound_class_method"


@staticmethod
def unbound_static_method():
    """subpackage.module2.unbound_static_method"""
    return "subpackage.module2.unbound_static_method"


class Class(object):

    """subpackage.module2.Class"""

    class Inner(object):

        """subpackage.module2.Class.Inner"""

        def method(self):
            """subpackage.module2.Class.Inner.method"""
            return "subpackage.module2.Class.Inner.method"


    @property
    def value(self):
        """subpackage.module2.Class.value.getter"""
        return "subpackage.module2.Class.value.getter"

    def method(self):
        """subpackage.module2.Class.method"""
        return "subpackage.module2.Class.method"

    @gorilla.patch(tests.utils.tomodule.Class, name='class_method2')
    @classmethod
    def class_method(cls):
        """subpackage.module2.Class.class_method"""
        return "subpackage.module2.Class.class_method"

    @gorilla.patch(tests.utils.tomodule.Class, name='static_method2')
    @staticmethod
    def static_method():
        """subpackage.module2.Class.static_method"""
        return "subpackage.module2.Class.static_method"



# -*- coding: utf-8 -*-

import gorilla

import tests.utils.tomodule


@gorilla.patch(tests.utils.tomodule, name='function1')
def function():
    """subpackage.module1.function"""
    return "subpackage.module1.function"


@gorilla.patch(tests.utils.tomodule.Class)
def unbound_method(self):
    """subpackage.module1.unbound_method"""
    return "subpackage.module1.unbound_method"


@gorilla.patch(tests.utils.tomodule.Class)
@classmethod
def unbound_class_method(cls):
    """subpackage.module1.unbound_class_method"""
    return "subpackage.module1.unbound_class_method"


@gorilla.patch(tests.utils.tomodule.Class)
@staticmethod
def unbound_static_method():
    """subpackage.module1.unbound_static_method"""
    return "subpackage.module1.unbound_static_method"


class Class(object):

    """subpackage.module1.Class"""

    class Inner(object):

        """subpackage.module1.Class.Inner"""

        def method(self):
            """subpackage.module1.Class.Inner.method"""
            return "subpackage.module1.Class.Inner.method"


    @gorilla.patch(tests.utils.tomodule.Class, name='value1')
    @property
    def value(self):
        """subpackage.module1.Class.value.getter"""
        return "subpackage.module1.Class.value.getter"

    @gorilla.patch(tests.utils.tomodule.Class, name='method1')
    def method(self):
        """subpackage.module1.Class.method"""
        return "subpackage.module1.Class.method"

    @classmethod
    def class_method(cls):
        """subpackage.module1.Class.class_method"""
        return "subpackage.module1.Class.class_method"

    @staticmethod
    def static_method():
        """subpackage.module1.Class.static_method"""
        return "subpackage.module1.Class.static_method"
